#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>
#include <memory.h>

#include <windows.h>
#include <crtdbg.h>

#include <Model.h>
#include <mqomaterial.h>
#include <mqoobject.h>

#define DBGH
#include <dbg.h>

#include <Bone.h>
#include <MQOFace.h>

#include <InfScope.h>
#include <MotionPoint.h>

#include <VecMath.h>

static int s_bonecnt = 0;

CBone::CBone() : m_curmp()
{
	InitParams();
	m_boneno = s_bonecnt;
	s_bonecnt++;
}

CBone::~CBone()
{
	DestroyObjs();
}

int CBone::InitParams()
{

	m_boneno = 0;
	m_topboneflag = 0;
	ZeroMemory( m_bonename, sizeof( char ) * 256 );
	ZeroMemory( m_vertpos, sizeof( D3DXVECTOR3 ) * BT_MAX );
	m_faceptr = 0;

	m_bonematerialno = -1;

	m_parent = 0;
	m_child = 0;
	m_brother = 0;

	m_isnum = 0;
	ZeroMemory( m_isarray, sizeof( CInfScope* ) * INFSCOPEMAX );

	return 0;
}
int CBone::DestroyObjs()
{

	return 0;
}

int CBone::ResetBoneCnt()
{
	s_bonecnt = 0;
	return 0;
}

int CBone::SetBoneTri( CMQOFace* srcface, D3DXVECTOR3* srcpoint )
{
	if( m_topboneflag == 0 ){
		sprintf_s( m_bonename, 256, srcface->m_bonename );

		if( srcface->m_bonetype == MIKOBONE_NORMAL ){
			*( m_vertpos + BT_PARENT ) = *( srcpoint + srcface->m_parentindex );
			*( m_vertpos + BT_CHILD ) = *( srcpoint + srcface->m_childindex );
			*( m_vertpos + BT_3RD ) = *( srcpoint + srcface->m_hindex );
		}else if( srcface->m_bonetype == MIKOBONE_FLOAT ){
			*( m_vertpos + BT_PARENT ) = *( srcpoint + srcface->m_parentindex );
			*( m_vertpos + BT_CHILD ) = *( srcpoint + srcface->m_childindex );
			*( m_vertpos + BT_3RD ) = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
		}else{
			ZeroMemory( m_vertpos, sizeof( D3DXVECTOR3 ) * BT_MAX );
			_ASSERT( 0 );
			return 1;
		}

		m_bonematerialno = srcface->m_materialno;

	}else{
		sprintf_s( m_bonename, 256, "TOP_%s", srcface->m_bonename );

		int bt;
		for( bt = 0; bt < BT_MAX; bt++ ){
			*( m_vertpos + bt ) = *( srcpoint + srcface->m_parentindex );
		}

		m_bonematerialno = -1;
	}
	m_faceptr = srcface;


	return 0;
}

int CBone::AddChild( CBone* childptr )
{
	childptr->m_parent = this;
	if( !m_child ){
		m_child = childptr;
	}else{
		CBone* broptr = m_child;
		while( broptr->m_brother ){
			broptr = broptr->m_brother;
		}
		broptr->m_brother = childptr;
	}

	return 0;
}

int CBone::AddInfScope( CInfScope* pinfscope )
{
	if( m_isnum >= INFSCOPEMAX ){
		_ASSERT( 0 );
		return 1;
	}

	m_isarray[ m_isnum ] = pinfscope;
	m_isnum++;

	return 0;
}

int CBone::CalcTargetCenter()
{
	int isno;
	for( isno = 0; isno < m_isnum; isno++ ){
		int setisno[INFSCOPEMAX];
		int setnum = 0;
		ZeroMemory( setisno, sizeof( int ) * INFSCOPEMAX );


		CInfScope* curis = m_isarray[ isno ];
		_ASSERT( curis );
		if( curis->m_settminmax != 0 ){
			continue;
		}

		setisno[ 0 ] = isno;
		setnum = 1;

		CMQOObject* curtarget = curis->m_targetobj;
		D3DXVECTOR3 tmpmin, tmpmax;
		tmpmin = curis->m_minv;
		tmpmax = curis->m_maxv;

		int isno2;
		for( isno2 = 0; isno2 < m_isnum; isno2++ ){
			CInfScope* calcis = m_isarray[ isno2 ];
			if( (isno2 != isno) && (calcis->m_settminmax == 0) && (calcis->m_targetobj == curtarget) ){

				if( setnum < INFSCOPEMAX ){
					setisno[setnum] = isno2;
					setnum++;
				}else{
					_ASSERT( 0 );
				}

				if( tmpmin.x > calcis->m_minv.x ){
					tmpmin.x = calcis->m_minv.x;
				}
				if( tmpmin.y > calcis->m_minv.y ){
					tmpmin.y = calcis->m_minv.y;
				}
				if( tmpmin.z > calcis->m_minv.z ){
					tmpmin.z = calcis->m_minv.z;
				}
				if( tmpmax.x < calcis->m_maxv.x ){
					tmpmax.x = calcis->m_maxv.x;
				}
				if( tmpmax.y < calcis->m_maxv.y ){
					tmpmax.y = calcis->m_maxv.y;
				}
				if( tmpmax.z < calcis->m_maxv.z ){
					tmpmax.z = calcis->m_maxv.z;
				}
			}
		}

		D3DXVECTOR3 tmpcenter, tmpdiff;
		float tmpmaxdist;
		tmpcenter = ( tmpmin + tmpmax ) * 0.5f;
		tmpdiff = tmpmin - tmpcenter;
		tmpmaxdist = D3DXVec3Length( &tmpdiff );

		int sno;
		for( sno = 0; sno < setnum; sno++ ){
			int setindex = setisno[ sno ];
			CInfScope* setis = m_isarray[ setindex ];
			setis->m_tminv = tmpmin;
			setis->m_tmaxv = tmpmax;
			setis->m_tcenter = tmpcenter;
			setis->m_tmaxdist = tmpmaxdist;
			setis->m_settminmax = 1;
		}
	}

	return 0;
}

int CBone::UpdateMatrix( D3DXMATRIX* wmat, D3DXMATRIX* parmat )
{
	CallF( m_curmp.UpdateMatrix( wmat, parmat, this ), return 1 );

	return 0;
}
int CBone::GetBoneMarkMatrix( D3DXMATRIX* srcworld, D3DXMATRIX* bmmatptr )
{
	D3DXVECTOR3 aftpar, aftchil;
	if( m_parent ){
		D3DXVec3TransformCoord( &aftpar, &(m_vertpos[BT_PARENT]), &( m_parent->m_curmp.m_worldmat ) );
	}else{
		D3DXVec3TransformCoord( &aftpar, &(m_vertpos[BT_PARENT]), srcworld );
	}
	D3DXVec3TransformCoord( &aftchil, &(m_vertpos[BT_CHILD]), &(m_curmp.m_worldmat) );

	D3DXVECTOR3 diffvec = aftchil - aftpar;
	float leng = DXVec3Length( &diffvec );
	if( (leng <= 0.000001f) || (m_vertpos[BT_PARENT] == m_vertpos[BT_CHILD]) ){
		ZeroMemory( bmmatptr, sizeof( D3DXMATRIX ) );
		return 0;
	}


	D3DXVECTOR3 basevec( 0.0f, 0.0f, 1.0f ); 
	D3DXVECTOR3 ndiffvec = diffvec / leng;
	CQuaternion rotq;
	rotq.RotationArc( basevec, ndiffvec );
	D3DXMATRIX rotmat = rotq.MakeRotMatX();


	D3DXMATRIX scalem;
	float scale;
	scale = leng / 50.0f;
	D3DXMatrixScaling( &scalem, scale, scale, scale );

	*bmmatptr = scalem * rotmat;

	bmmatptr->_41 += aftpar.x;
	bmmatptr->_42 += aftpar.y;
	bmmatptr->_43 += aftpar.z;

	return 0;
}


int CBone::GetBoneCircleMatrix( D3DXMATRIX* bcmatptr )
{
	*bcmatptr = m_curmp.m_worldmat;

	return 0;
}
