#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>
#include <memory.h>

#include <windows.h>
#include <crtdbg.h>

#include <InfScope.h>

#include <Model.h>
#include <mqomaterial.h>
#include <mqoobject.h>

#define DBGH
#include <dbg.h>

#include <Bone.h>
#include <MQOFace.h>

#include <VecMath.h>

static int s_alloccnt = 0;

CInfScope::CInfScope()
{
	InitParams();
	s_alloccnt++;
	m_serialno = s_alloccnt;
}

CInfScope::~CInfScope()
{
	DestroyObjs();
}

int CInfScope::InitParams()
{
	m_serialno = 0;
	m_materialno = 0;
	m_validflag = 0;

	m_facenum = 0;
	m_ppface = 0;

	m_vnum = 0;
	m_pvert = 0;

	m_applyboneno = -1;
	m_targetobj = 0;

	m_minv = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_maxv = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_center = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );

	m_settminmax = 0;
	m_tminv = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_tmaxv = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_tcenter = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_tmaxdist = 1.0f;

	m_tmpappbone = 0;

	return 0;
}
int CInfScope::DestroyObjs()
{
	if( m_ppface ){
		free( m_ppface );
		m_ppface = 0;
	}
	m_facenum = 0;

	InitParams();

	return 0;
}

int CInfScope::CalcCenter()
{
	if( (m_facenum <= 0) || !m_ppface || (m_vnum <= 0) || !m_pvert ){
		return 0;
	}

	D3DXVECTOR3 minv( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 maxv( 0.0f, 0.0f, 0.0f );

	int firstflag = 1;

	int fno;
	for( fno = 0; fno < m_facenum; fno++ ){
		CMQOFace* curface = *( m_ppface + fno );
		int fi;
		for( fi = 0; fi < curface->m_pointnum; fi++ ){
			D3DXVECTOR3* chkv = m_pvert + curface->m_index[ fi ];

			if( firstflag == 1 ){
				minv = *chkv;
				maxv = *chkv;
				firstflag = 0;
			}else{
				if( minv.x > chkv->x ){
					minv.x = chkv->x;
				}
				if( minv.y > chkv->y ){
					minv.y = chkv->y;
				}
				if( minv.z > chkv->z ){
					minv.z = chkv->z;
				}

				if( maxv.x < chkv->x ){
					maxv.x = chkv->x;
				}
				if( maxv.y < chkv->y ){
					maxv.y = chkv->y;
				}
				if( maxv.z < chkv->z ){
					maxv.z = chkv->z;
				}
			}			
		}
	}

	m_minv = minv;
	m_maxv = maxv;
	m_center = ( minv + maxv ) * 0.5f;

	return 0;
}

int CInfScope::CheckInCnt( D3DXVECTOR3* vptr, int* incntptr, int* justptr )
{
	*incntptr = 0;
	*justptr = 0;

	int fno;
	for( fno = 0; fno < m_facenum; fno++ ){
		CMQOFace* curface = *( m_ppface + fno );
		if( curface->m_pointnum == 3 ){
			(*incntptr) += ChkRay( curface->m_index[0], curface->m_index[1], curface->m_index[2], vptr, justptr );
		}else if( curface->m_pointnum == 4 ){
			(*incntptr) += ChkRay( curface->m_index[0], curface->m_index[1], curface->m_index[2], vptr, justptr );
			(*incntptr) += ChkRay( curface->m_index[0], curface->m_index[2], curface->m_index[3], vptr, justptr );
		}
		if( (*justptr) > 0 ){
			break;
		}
	}

	return 0;
}


int CInfScope::ChkRay( int i1, int i2, int i3, D3DXVECTOR3* vptr, int* justptr )
{
	if( (i1 >= m_vnum) || (i2 >= m_vnum) || (i3 >= m_vnum) ){
		_ASSERT( 0 );
		return 1;
	}


	D3DXVECTOR3 v1;
	v1 = *vptr;

	D3DXVECTOR3 v;
//	v = D3DXVECTOR3( 0.999f, 1.234f, -1.567f );
	v = D3DXVECTOR3( 0.987f, 1.765f, -1.567f );

	D3DXVECTOR3 e;
	DXVec3Normalize( &e, &v );

	D3DXVECTOR3 s, t;
	s = *( m_pvert + i2 ) - *( m_pvert + i1 );
	t = *( m_pvert + i3 ) - *( m_pvert + i1 );
	D3DXVECTOR3 abc;
	DXVec3Cross( &abc, &s, &t );
	DXVec3Normalize( &abc, &abc );

	float d;
	d = -DXVec3Dot( &abc, (m_pvert + i1) );

	if( DXVec3Dot( &abc, &e ) == 0.0f ){
		_ASSERT( 0 );
		return 0;
	}

	float k;
	k = -( (DXVec3Dot( &abc, &v1 ) + d) / DXVec3Dot( &abc, &e ) );
	if( fabs( k ) <= 0.10f ){
		(*justptr)++;
		return 0;
	}
	if( k < 0.0f ){
		return 0;
	}

	D3DXVECTOR3 q;
	q = v1 + k * e;

	D3DXVECTOR3 g0, g1, cA, cB, cC;
	
	g1 = *(m_pvert + i2) - *(m_pvert + i1);
	g0 = q - *(m_pvert + i1);
	DXVec3Cross( &cA, &g0, &g1 );
	DXVec3Normalize( &cA, &cA );

	g1 = *(m_pvert + i3) - *(m_pvert + i2);
	g0 = q - *(m_pvert + i2);
	DXVec3Cross( &cB, &g0, &g1 );
	DXVec3Normalize( &cB, &cB );

	g1 = *(m_pvert + i1) - *(m_pvert + i3);
	g0 = q - *(m_pvert + i3);
	DXVec3Cross( &cC, &g0, &g1 );
	DXVec3Normalize( &cC, &cC );


	float dota, dotb, dotc;
	dota = DXVec3Dot( &abc, &cA );
	dotb = DXVec3Dot( &abc, &cB );
	dotc = DXVec3Dot( &abc, &cC );

	int zeroflag;
	zeroflag = (fabs(dota) < 0.05f) && (fabs(dotb) < 0.05f) && (fabs(dotc) < 0.05f);
	if( zeroflag != 0 ){
		//(*justptr)++;
		return 1;
	}

	if( ((dota <= -0.50f) && (dotb <= -0.50f) && (dotc <= -0.50f)) ||
		((dota >= 0.50f) && (dotb >= 0.50f) && (dotc >= 0.50f)) ){
		return 1;
	}else{
		return 0;
	}

}


