//#include <stdafx.h>

#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>
#include <memory.h>

#include <windows.h>
#include <crtdbg.h>

#include <Model.h>
#include <polymesh3.h>
#include <ExtLine.h>

#include <GetMaterial.h>

#include <mqofile.h>
#include <mqomaterial.h>
#include <mqoobject.h>
#include <Bone.h>
#include <mqoface.h>

#define DBGH
#include <dbg.h>

#include <DXUT.h>
#include <DXUTcamera.h>
#include <DXUTgui.h>
#include <DXUTsettingsdlg.h>
#include <SDKmisc.h>

#include <DispObj.h>
#include <InfScope.h>
#include <MySprite.h>

static int s_alloccnt = 0;

extern WCHAR g_basedir[ MAX_PATH ];
extern ID3DXEffect*	g_pEffect;
extern D3DXHANDLE g_hmWMatArray;
extern D3DXHANDLE g_hmWorld;


int g_dbgflag = 0;

CModel::CModel()
{
	InitParams();
	s_alloccnt++;
	m_modelno = s_alloccnt;
}
CModel::~CModel()
{
	DestroyObjs();
}
int CModel::InitParams()
{
	m_modelno = 0;
	m_pdev = 0;

	ZeroMemory( m_filename, sizeof( WCHAR ) * MAX_PATH );
	ZeroMemory( m_dirname, sizeof( WCHAR ) * MAX_PATH );

	ZeroMemory( m_bonemat, sizeof( D3DXMATRIX ) * MAXBONENUM );
	D3DXMatrixIdentity( &m_matWorld );
	D3DXMatrixIdentity( &m_matVP );

	return 0;
}
int CModel::DestroyObjs()
{
	DestroyMaterial();
	DestroyObject();
	DestroyAncObj();

	InitParams();

	return 0;
}

int CModel::DestroyMaterial()
{
	map<int, CMQOMaterial*>::iterator itr;
	for( itr = m_material.begin(); itr != m_material.end(); itr++ ){
		CMQOMaterial* delmat = itr->second;
		if( delmat ){
			delete delmat;
		}
	}
	m_material.erase( m_material.begin(), m_material.end() );

	return 0;
}
int CModel::DestroyObject()
{

	map<int, CMQOObject*>::iterator itr;
	for( itr = m_object.begin(); itr != m_object.end(); itr++ ){
		CMQOObject* delobj = itr->second;
		if( delobj ){
			delete delobj;
		}
	}
	m_object.erase( m_object.begin(), m_object.end() );

	return 0;
}

int CModel::DestroyAncObj()
{
	map<int, CMQOMaterial*>::iterator itrmat;
	for( itrmat = m_ancmaterial.begin(); itrmat != m_ancmaterial.end(); itrmat++ ){
		CMQOMaterial* delmat = itrmat->second;
		if( delmat ){
			delete delmat;
		}
	}
	m_ancmaterial.erase( m_ancmaterial.begin(), m_ancmaterial.end() );

	map<int, CMQOObject*>::iterator itrobj;
	for( itrobj = m_ancobject.begin(); itrobj != m_ancobject.end(); itrobj++ ){
		CMQOObject* delobj = itrobj->second;
		if( delobj ){
			delete delobj;
		}
	}
	m_ancobject.erase( m_ancobject.begin(), m_ancobject.end() );


	map<int, CMQOObject*>::iterator itrobj2;
	for( itrobj2 = m_boneobject.begin(); itrobj2 != m_boneobject.end(); itrobj2++ ){
		CMQOObject* delobj = itrobj2->second;
		if( delobj ){
			delete delobj;
		}
	}
	m_boneobject.erase( m_boneobject.begin(), m_boneobject.end() );


	map<int, CBone*>::iterator itrbone;
	for( itrbone = m_bonelist.begin(); itrbone != m_bonelist.end(); itrbone++ ){
		CBone* delbone = itrbone->second;
		if( delbone ){
			if( delbone->m_topboneflag == 0 ){
				delete delbone;
			}
		}
	}
	m_bonelist.erase( m_bonelist.begin(), m_bonelist.end() );

	if( m_topbone ){
		delete [] m_topbone;
		m_topbone = 0;
	}
	m_topbonenum = 0;


	map<int, CInfScope*>::iterator itris;
	for( itris = m_infscope.begin(); itris != m_infscope.end(); itris++ ){
		CInfScope* delis = itris->second;
		if( delis ){
			delete delis;
		}
	}
	m_infscope.erase( m_infscope.begin(), m_infscope.end() );

	m_objectname.erase( m_objectname.begin(), m_objectname.end() );
	m_bonename.erase( m_bonename.begin(), m_bonename.end() );

	return 0;
}

int CModel::LoadMQO( LPDIRECT3DDEVICE9 pdev, WCHAR* wfile, int ismedia )
{
	m_pdev = pdev;

	WCHAR fullname[MAX_PATH];

	if( ismedia == 1 ){
		WCHAR str[MAX_PATH];
		HRESULT hr;
		hr = DXUTFindDXSDKMediaFileCch( str, MAX_PATH, wfile );
		if( hr != S_OK ){
			::MessageBoxA( NULL, "media not found error !!!", "load error", MB_OK );
			_ASSERT( 0 );
			return 1;
		}
		wcscpy_s( fullname, MAX_PATH, g_basedir );
		wcscat_s( fullname, MAX_PATH, str );

	}else{
		wcscpy_s( fullname, MAX_PATH, wfile );
	}

    WCHAR* strLastSlash = NULL;
    strLastSlash = wcsrchr( fullname, TEXT( '\\' ) );
    if( strLastSlash )
    {
		*strLastSlash = 0;
		wcscpy_s( m_dirname, MAX_PATH, fullname );
		wcscpy_s( m_filename, MAX_PATH, strLastSlash + 1 );
	}else{
		ZeroMemory( m_dirname, sizeof( WCHAR ) * MAX_PATH );
		wcscpy_s( m_filename, MAX_PATH, fullname );
	}


	SetCurrentDirectory( m_dirname );


	DestroyMaterial();
	DestroyObject();

	CMQOFile mqofile;
	D3DXVECTOR3 op( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 or( 0.0f, 0.0f, 0.0f );
	CallF( mqofile.LoadMQOFile( m_pdev, 1.0f, m_filename, op, or, this ), return 1 );

	CallF( MakeObjectName(), return 1 );


	CallF( CreateMaterialTexture(), return 1 );


	return 0;
}

int CModel::CreateMaterialTexture()
{
	map<int,CMQOMaterial*>::iterator itr;
	for( itr = m_material.begin(); itr != m_material.end(); itr++ ){
		CMQOMaterial* curmat = itr->second;
		CallF( curmat->CreateTexture( m_dirname ), return 1 );
	}

	return 0;
}

int CModel::OnRender( LPDIRECT3DDEVICE9 pdev, int lightflag, D3DXVECTOR4 diffusemult, CModel* bmarkptr, CMySprite* bcircleptr )
{
	map<int,CMQOObject*>::iterator itr;
	for( itr = m_object.begin(); itr != m_object.end(); itr++ ){
		CMQOObject* curobj = itr->second;
		if( curobj ){
			if( curobj->m_dispobj ){
				CallF( curobj->m_dispobj->RenderNormal( lightflag, diffusemult ), return 1 );
			}
			if( curobj->m_displine ){
				CallF( curobj->m_displine->RenderLine( diffusemult ), return 1 );
			}
		}
	}

	if( bmarkptr || bcircleptr ){
		CallF( RenderBoneMark( pdev, bmarkptr, bcircleptr ), return 1 );
	}

	return 0;
}

int CModel::GetModelBound( MODELBOUND* dstb )
{
	MODELBOUND mb;
	MODELBOUND addmb;
	ZeroMemory( &mb, sizeof( MODELBOUND ) );

	int calcflag = 0;
	map<int,CMQOObject*>::iterator itr;
	for( itr = m_object.begin(); itr != m_object.end(); itr++ ){
		CMQOObject* curobj = itr->second;
		if( curobj->m_pm3 && curobj->m_pm3->m_infbone ){
			curobj->m_pm3->CalcBound();
			if( calcflag == 0 ){
				mb = curobj->m_pm3->m_bound;
			}else{
				addmb = curobj->m_pm3->m_bound;
				AddModelBound( &mb, &addmb );
			}
			calcflag++;
		}
		if( curobj->m_extline ){
			curobj->m_extline->CalcBound();
			if( calcflag == 0 ){
				mb = curobj->m_extline->m_bound;
			}else{
				addmb = curobj->m_extline->m_bound;
				AddModelBound( &mb, &addmb );
			}
			calcflag++;
		}
	}

	*dstb = mb;

	return 0;
}

int CModel::AddModelBound( MODELBOUND* mb, MODELBOUND* addmb )
{
	D3DXVECTOR3 newmin = mb->min;
	D3DXVECTOR3 newmax = mb->max;

	if( newmin.x > addmb->min.x ){
		newmin.x = addmb->min.x;
	}
	if( newmin.y > addmb->min.y ){
		newmin.y = addmb->min.y;
	}
	if( newmin.z > addmb->min.z ){
		newmin.z = addmb->min.z;
	}

	if( newmax.x < addmb->max.x ){
		newmax.x = addmb->max.x;
	}
	if( newmax.y < addmb->max.y ){
		newmax.y = addmb->max.y;
	}
	if( newmax.z < addmb->max.z ){
		newmax.z = addmb->max.z;
	}

	mb->center = ( newmin + newmax ) * 0.5f;
	mb->min = newmin;
	mb->max = newmax;

	D3DXVECTOR3 diff;
	diff = mb->center - newmin;
	mb->r = D3DXVec3Length( &diff );

	return 0;
}

int CModel::MakeBoneTri()
{
	if( m_boneobject.begin() == m_boneobject.end() ){
		m_topbonenum = 0;
		m_topbone = 0;
		return 0;//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	}


	CMQOFace* toplevelface[ MAXBONENUM ];
	int toplevelnum = 0;

	map<int, CMQOObject*>::iterator itrobj;
	itrobj = m_boneobject.begin();
	CMQOObject* curobj = itrobj->second;
	if( curobj ){
		CallF( curobj->SetMikoBoneIndex3(), return 1 );
		CallF( curobj->SetMikoBoneIndex2(), return 1 );
		CallF( curobj->CheckSameMikoBone(), return 1 );
		CallF( curobj->GetTopLevelMikoBone( toplevelface, &toplevelnum, MAXBONENUM ), return 1 );

		CallF( curobj->InitFaceDirtyFlag(), return 1 );
		int topno;
		for( topno = 0; topno < toplevelnum; topno++ ){
			CallF( curobj->SetTreeMikoBone( toplevelface[ topno ] ), return 1 );
		}

		CallF( curobj->InitFaceDirtyFlag(), return 1 );
		int islooped = 0;
		int jointnum = 0;
		int totaljointnum = 0;
		for( topno = 0; topno < toplevelnum; topno++ ){
			jointnum = 0;
			CallF( curobj->CheckLoopedMikoBoneReq( toplevelface[ topno ], &islooped, &jointnum ), return 1 );
			if( islooped ){
				::MessageBox( NULL, L"[v͏dĂ{[\܂B(딻肩ij)ǂݍ߂܂B", L"G[", MB_OK );
				_ASSERT( 0 );
				return 1;
			}
			totaljointnum += jointnum;
			if( totaljointnum >= MAXBONENUM ){
				::MessageBox( NULL, L"{[̐܂B43ȉɂĂB", L"G[", MB_OK );
				_ASSERT( 0 );
				return 1;
			}
		}

		CallF( curobj->SetMikoBoneName( m_ancmaterial ), return 1 );
		CallF( curobj->SetMikoFloatBoneName(), return 1 );


		CBone::ResetBoneCnt();
		m_topbonenum = toplevelnum;
		m_topbone = new CBone[ m_topbonenum ];
		if( !m_topbone ){
			_ASSERT( 0 );
			return 1;
		}

		int errcnt = 0;
		for( topno = 0; topno < toplevelnum; topno++ ){
			CBone* topbone = m_topbone + topno;
			CMQOFace* topface = toplevelface[ topno ];
			topbone->m_topboneflag = 1;
			CallF( topbone->SetBoneTri( topface, curobj->m_pointbuf ), return 1 );
			m_bonelist[topbone->m_boneno] = topbone;


			MakeBoneReq( topbone, topface, curobj->m_pointbuf, 0, &errcnt );
			if( errcnt > 0 ){
				_ASSERT( 0 );
				return 1;
			}
		}
	}



	return 0;
}

void CModel::MakeBoneReq( CBone* parbone, CMQOFace* curface, D3DXVECTOR3* pointptr, int broflag, int* errcntptr )
{
	if( *errcntptr > 0 ){
		return;
	}

	if( (curface->m_bonetype != MIKOBONE_NORMAL) && (curface->m_bonetype != MIKOBONE_FLOAT) ){
		return;
	}

	CBone* newbone = new CBone();
	if( !newbone ){
		_ASSERT( 0 );
		(*errcntptr)++;
		return;
	}
	m_bonelist[ newbone->m_boneno ] = newbone;

	int ret;
	ret = newbone->SetBoneTri( curface, pointptr );
	if( ret ){
		_ASSERT( 0 );
		(*errcntptr)++;
		return;
	}

	ret = parbone->AddChild( newbone );
	if( ret ){
		_ASSERT( 0 );
		(*errcntptr)++;
		return;
	}

	char* bonename = newbone->m_bonename;
	int nameleng = (int)strlen( bonename );
	string strname( bonename, bonename + nameleng );
	m_bonename[ strname ] = newbone;


/////////
	if( curface->m_child ){
		MakeBoneReq( newbone, curface->m_child, pointptr, 1, errcntptr );
	}
	if( broflag && curface->m_brother ){
		MakeBoneReq( parbone, curface->m_brother, pointptr, 1, errcntptr );
	}
}

int CModel::MakeInfScope()
{
	map<int, CMQOObject*>::iterator itrobj;
	for( itrobj = m_ancobject.begin(); itrobj != m_ancobject.end(); itrobj++ ){
		CMQOObject* curobj = itrobj->second;
		if( curobj ){
			int matnum = 0;
			CallF( curobj->GetMaterialNoInUse( 0, 0, &matnum ), return 1 );

			if( matnum > 0 ){
				CMQOObject* targetobjptr = 0;
				CBone* applynamebone = 0;

				char* ancnameptr = curobj->m_name;
				char* sepptr = 0;
				sepptr = strchr( ancnameptr, '|' );
				if( sepptr ){
					char* startptr = sepptr + 1;
					int leng = (int)strlen( startptr );
					string findname( startptr, startptr + leng );
					targetobjptr = m_objectname[ findname ];
				}



				int* matarray = (int*)malloc( sizeof( int ) * matnum );
				if( !matarray ){
					_ASSERT( 0 );
					return 1;
				}
				ZeroMemory( matarray, sizeof( int ) * matnum );

				int chknum = 0;
				CallF( curobj->GetMaterialNoInUse( matarray, matnum, &chknum ), return 1 );

				int matcnt;
				for( matcnt = 0; matcnt < matnum; matcnt++ ){
					int matno = *( matarray + matcnt );

					int facenum = 0;
					CallF( curobj->GetFaceInMaterial( matno, 0, 0, &facenum ), return 1 );
					if( facenum > 0 ){
						CMQOFace** ppface = (CMQOFace**)malloc( sizeof( CMQOFace* ) * facenum );
						ZeroMemory( ppface, sizeof( CMQOFace* ) * facenum );
						int chkfnum = 0;
						CallF( curobj->GetFaceInMaterial( matno, ppface, facenum, &chkfnum ), return 1 );

						int setfacenum = 0;
						int setfirst = 1;
						int shapeno = 0;
						while( (setfirst == 1) || (setfacenum > 0) ){
							setfacenum = 0;

							CMQOFace* startface = 0;
							int chkf;
							for( chkf = 0; chkf < facenum; chkf++ ){
								if( (*( ppface + chkf ))->m_shapeno == -1 ){
									startface = *(ppface + chkf);
									break;
								}
							}
							if( startface ){
								int si;
								for( si = 0; si < startface->m_pointnum; si++ ){
									int searchp = startface->m_index[ si ];
									CallF( SetShapeNoReq( ppface, facenum, searchp, shapeno, &setfacenum), return 1 );
								}
								if( setfacenum > 0 ){
									CMQOFace** ppface2 = (CMQOFace**)malloc( sizeof( CMQOFace* ) * setfacenum );
									if( !ppface2 ){
										_ASSERT( 0 );
										return 1;
									}
									CallF( SetFaceOfShape( ppface, facenum, shapeno, ppface2, setfacenum), return 1 );

									CInfScope* newis = new CInfScope();
									if( !newis ){
										_ASSERT( 0 );
										return 1;
									}
									newis->m_materialno = matno;
									newis->m_facenum = setfacenum;
									newis->m_ppface = ppface2;
									newis->m_vnum = curobj->m_vertex;
									newis->m_pvert = curobj->m_pointbuf;
									newis->m_targetobj = targetobjptr;
									if( applynamebone ){
										newis->m_tmpappbone = applynamebone;
									}
									m_infscope[ newis->m_serialno ] = newis;

									CallF( newis->CalcCenter(), return 1 );

									shapeno++;

								}
			
							}else{
								break;
							}

							setfirst = 0;
						}
						free( ppface );
					}
				}
				free( matarray );
			}
		}
	}


	D3DXVECTOR3 bonediff;
	float bonedist, tmpdist;

	map<int, CInfScope*>::iterator itris;
	for( itris = m_infscope.begin(); itris != m_infscope.end(); itris++ ){
		CInfScope* curis = itris->second;
		if( curis->m_tmpappbone == 0 ){
			CMQOMaterial* curmaterial = GetMaterialFromNo( m_ancmaterial, curis->m_materialno );
			if( curmaterial ){
				int setboneno = -1;
				map<int, CBone*>::iterator itrbone;
				for( itrbone = m_bonelist.begin(); itrbone != m_bonelist.end(); itrbone++ ){
					CBone* cmpbone = itrbone->second;
					if( cmpbone->m_bonematerialno == curmaterial->materialno ){
						if( setboneno == -1 ){
							setboneno = cmpbone->m_boneno;
							bonediff = (cmpbone->m_vertpos[BT_PARENT] + cmpbone->m_vertpos[BT_CHILD]) * 0.5f - curis->m_center;
							bonedist = D3DXVec3Length( &bonediff );
						}else{
							bonediff = (cmpbone->m_vertpos[BT_PARENT] + cmpbone->m_vertpos[BT_CHILD]) * 0.5f - curis->m_center;
							tmpdist = D3DXVec3Length( &bonediff );
							if( tmpdist < bonedist ){
								setboneno = cmpbone->m_boneno;
								bonedist = tmpdist;
							}
						}
					}
				}

				if( setboneno >= 0 ){
					curis->m_applyboneno = setboneno;
					curis->m_validflag = 1;
		
					CallF( m_bonelist[ setboneno ]->AddInfScope( curis ), return 1 );
				}
			}
		}else{
			curis->m_applyboneno = curis->m_tmpappbone->m_boneno;
			curis->m_validflag = 1;

			CallF( m_bonelist[ curis->m_applyboneno ]->AddInfScope( curis ), return 1 );

		}
	}

	map<int, CBone*>::iterator itrb;
	for( itrb = m_bonelist.begin(); itrb != m_bonelist.end(); itrb++ ){
		CBone* curbone = itrb->second;
		if( curbone ){
			CallF( curbone->CalcTargetCenter(), return 1 );
		}
	}

	return 0;
}


int CModel::CalcInf()
{
	if( m_topbonenum <= 0 ){
		return 0;
	}

	map<int, CMQOObject*>::iterator itrobj;
	for( itrobj = m_object.begin(); itrobj != m_object.end(); itrobj++ ){
		CMQOObject* curobj = itrobj->second;
		if( curobj && curobj->m_pm3 ){
			CBone* applybone = 0;
			char* nameptr = curobj->m_name;
			char* applyptr = strchr( nameptr, '-' );
			if( applyptr ){
				int leng = (int)strlen( applyptr + 1 );
				string strapply( applyptr + 1, applyptr + 1 + leng );
				applybone = m_bonename[ strapply ];
			}

			if( applybone ){
				CallF( curobj->m_pm3->CalcInfNoSkin( applybone ), return 1 );
			}else{
				CallF( curobj->m_pm3->CalcInf( curobj, m_bonelist ), return 1; );
			}
		}
	}


	return 0;
}


int CModel::SetShapeNoReq( CMQOFace** ppface, int facenum, int searchp, int shapeno, int* setfacenum )
{

	int fno;
	CMQOFace* findface[200];
	ZeroMemory( findface, sizeof( CMQOFace* ) * 200 );
	int findnum = 0;

	for( fno = 0; fno < facenum; fno++ ){
		CMQOFace* curface = *( ppface + fno );
		if( curface->m_shapeno != -1 ){
			continue;
		}

		int chki;
		for( chki = 0; chki < curface->m_pointnum; chki++ ){
			if( searchp == curface->m_index[ chki ] ){
				if( findnum >= 200 ){
					_ASSERT( 0 );
					return 1;
				}
				curface->m_shapeno = shapeno;
				findface[findnum] = curface;
				findnum++;
				break;
			}
		}
	}

	if( findnum > 0 ){
		(*setfacenum) += findnum;

		int findno;
		for( findno = 0; findno < findnum; findno++ ){
			CMQOFace* fface = findface[ findno ];
			int i;
			for( i = 0; i < fface->m_pointnum; i++ ){
				int newsearch = fface->m_index[ i ];
				if( newsearch != searchp ){
					SetShapeNoReq( ppface, facenum, newsearch, shapeno, setfacenum );
				}
			}
		}
	}
	return 0;
}

int CModel::SetFaceOfShape( CMQOFace** ppface, int facenum, int shapeno, CMQOFace** ppface2, int setfacenum )
{
	int setno = 0;
	int fno;
	for( fno = 0; fno < facenum; fno++ ){
		CMQOFace* curface = *( ppface + fno );
		if( curface->m_shapeno == shapeno ){
			if( setno >= setfacenum ){
				_ASSERT( 0 );
				return 1;
			}
			*( ppface2 + setno ) = curface;
			setno++;
		}
	}

	_ASSERT( setno == setfacenum );

	return 0;
}

int CModel::MakeObjectName()
{
	map<int, CMQOObject*>::iterator itrobj;
	for( itrobj = m_object.begin(); itrobj != m_object.end(); itrobj++ ){
		CMQOObject* curobj = itrobj->second;
		if( curobj ){
			char* nameptr = curobj->m_name;
			int sdefcmp, bdefcmp;
			sdefcmp = strncmp( nameptr, "sdef:", 5 );
			bdefcmp = strncmp( nameptr, "bdef:", 5 );
			if( (sdefcmp != 0) && (bdefcmp != 0) ){
				int leng = (int)strlen( nameptr );
				string firstname( nameptr, nameptr + leng );
				m_objectname[ firstname ] = curobj;
				curobj->m_dispname = firstname;
			}else{
				char* startptr = nameptr + 5;
				int leng = (int)strlen( startptr );
				string firstname( startptr, startptr + leng );
				m_objectname[ firstname ] = curobj;
				curobj->m_dispname = firstname;
			}
		}
	}

	return 0;
}

int CModel::DbgDump()
{
	DbgOut( L"######start DbgDump\r\n" );

	DbgOut( L"Dump Bone And InfScope %d\r\n", m_topbonenum );

	int topno;
	for( topno = 0; topno < m_topbonenum; topno++ ){
		DbgOut( L"topbone %d tree start\r\n", topno );
		CBone* topbone = m_topbone + topno;
		DbgDumpBoneReq( topbone, 0 );
	}

//	MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, name, 256, wname, 256 );

	DbgOut( L"######end DbgDump\r\n" );


	return 0;
}

int CModel::DbgDumpBoneReq( CBone* boneptr, int broflag )
{
	char mes[1024];
	WCHAR wmes[1024];

	sprintf_s( mes, 1024, "\tboneno %d, bonename %s\r\n", boneptr->m_boneno, boneptr->m_bonename );
	MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, mes, 1024, wmes, 1024 );
	DbgOut( wmes );

	DbgOut( L"\t\tbonepos (%f, %f, %f), (%f, %f, %f)\r\n", 
		boneptr->m_vertpos[ BT_PARENT ].x, boneptr->m_vertpos[ BT_PARENT ].y, boneptr->m_vertpos[ BT_PARENT ].z,
		boneptr->m_vertpos[ BT_CHILD ].x, boneptr->m_vertpos[ BT_CHILD ].y, boneptr->m_vertpos[ BT_CHILD ].z );

	
	DbgOut( L"\t\tinfscopenum %d\r\n", boneptr->m_isnum );
	int isno;
	for( isno = 0; isno < boneptr->m_isnum; isno++ ){
		CInfScope* curis = boneptr->m_isarray[ isno ];
		CBone* infbone = 0;
		if( curis->m_applyboneno >= 0 ){
			infbone = m_bonelist[ curis->m_applyboneno ];
			sprintf_s( mes, 1024, "\t\tInfScope %d, validflag %d, facenum %d, applybone %s\r\n", 
				isno, curis->m_validflag, curis->m_facenum, infbone->m_bonename );
			MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, mes, 1024, wmes, 1024 );
			DbgOut( wmes );
		}else{
			DbgOut( L"\t\tInfScope %d, validflag %d, facenum %d, applybone is none\r\n", 
				isno, curis->m_validflag, curis->m_facenum );
		}

		if( curis->m_targetobj ){
			sprintf_s( mes, 1024, "\t\t\ttargetobj %s\r\n", curis->m_targetobj->m_name );
			MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, mes, 1024, wmes, 1024 );
			DbgOut( wmes );
		}else{
			DbgOut( L"\t\t\ttargetobj is none\r\n" );
		}
	}

//////////
	if( boneptr->m_child ){
		DbgDumpBoneReq( boneptr->m_child, 1 );
	}
	if( (broflag == 1) && boneptr->m_brother ){
		DbgDumpBoneReq( boneptr->m_brother, 1 );
	}



	return 0;
}

int CModel::MakePolyMesh3()
{
	map<int,CMQOObject*>::iterator itr;
	for( itr = m_object.begin(); itr != m_object.end(); itr++ ){
		CMQOObject* curobj = itr->second;
		if( curobj ){
			CallF( curobj->MakePolymesh3( m_pdev, m_material ), return 1 );
		}
	}

	return 0;
}

int CModel::MakeExtLine()
{
	map<int,CMQOObject*>::iterator itr;
	for( itr = m_object.begin(); itr != m_object.end(); itr++ ){
		CMQOObject* curobj = itr->second;
		if( curobj ){
			CallF( curobj->MakeExtLine(), return 1 );
		}
	}

	return 0;
}

int CModel::MakeDispObj()
{
	int hasbone;
	if( m_topbonenum <= 0 ){
		hasbone = 0;
	}else{
		hasbone = 1;
	}

	map<int,CMQOObject*>::iterator itr;
	for( itr = m_object.begin(); itr != m_object.end(); itr++ ){
		CMQOObject* curobj = itr->second;
		if( curobj ){
			CallF( curobj->MakeDispObj( m_pdev, m_material, hasbone ), return 1 );
		}
	}

	return 0;
}

int CModel::UpdateMatrix( D3DXMATRIX* wmat, D3DXMATRIX* vpmat )
{
	m_matWorld = *wmat;
	m_matVP = *vpmat;

	int topno;
	for( topno = 0; topno < m_topbonenum; topno++ ){
		CBone* topbone = m_topbone + topno;
		D3DXMATRIX inimat;
		D3DXMatrixIdentity( &inimat );

		UpdateMatrixReq( wmat, &inimat, topbone, 0 );
	}


//	int UpdateMatrix( D3DXMATRIX* wmat, D3DXMATRIX* parmat, CBone* srcbone );

	return 0;
}

void CModel::UpdateMatrixReq( D3DXMATRIX* wmat, D3DXMATRIX* parmat, CBone* srcbone, int broflag )
{
	if( srcbone && wmat && parmat ){
		srcbone->UpdateMatrix( wmat, parmat );
	}else{
		return;
	}

	if( srcbone->m_child ){
		UpdateMatrixReq( wmat, &(srcbone->m_curmp.m_totalmat), srcbone->m_child, 1 );
	}
	if( srcbone->m_brother && (broflag == 1) ){
		UpdateMatrixReq( wmat, parmat, srcbone->m_brother, 1 );
	}
}

int CModel::SetShaderConst()
{
	map<int, CBone*>::iterator itrbone;
	int setcnt = 0;
	for( itrbone = m_bonelist.begin(); itrbone != m_bonelist.end(); itrbone++ ){
		int boneno = itrbone->first;
		CBone* curbone = itrbone->second;
		if( curbone && (boneno >= 0) && (boneno < MAXBONENUM) ){
			m_bonemat[boneno] = curbone->m_curmp.m_worldmat;
			setcnt++;
		}
	}

	if( setcnt > 0 ){
		HRESULT hr;
		hr = g_pEffect->SetMatrixArray( g_hmWMatArray, m_bonemat, MAXBONENUM );
		if( hr != D3D_OK ){
			_ASSERT( 0 );
			return 1;
		}
	}
	return 0;
}

int CModel::SetBoneEul( int boneno, D3DXVECTOR3 srceul )
{
	if( boneno < 0 ){
		_ASSERT( 0 );
		return 1;
	}

	CBone* curbone;
	curbone = m_bonelist[ boneno ];
	if( curbone ){
		curbone->m_curmp.SetEul( srceul );
	}

	return 0;
}

int CModel::GetBoneEul( int boneno, D3DXVECTOR3* dsteul )
{
	if( boneno < 0 ){
		_ASSERT( 0 );
		return 1;
	}

	CBone* curbone;
	curbone = m_bonelist[ boneno ];
	if( curbone ){
		*dsteul = curbone->m_curmp.m_eul;
	}else{
		*dsteul = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	}

	return 0;
}

int CModel::RenderBoneMark( LPDIRECT3DDEVICE9 pdev, CModel* bmarkptr, CMySprite* bcircleptr )
{
	if( m_topbonenum <= 0 ){
		return 0;
	}

	pdev->SetRenderState( D3DRS_ZFUNC, D3DCMP_ALWAYS );

	if( bmarkptr ){
		map<int, CBone*>::iterator itrbone;
		for( itrbone = m_bonelist.begin(); itrbone != m_bonelist.end(); itrbone++ ){
			CBone* boneptr = itrbone->second;
			if( boneptr ){

				D3DXMATRIX bmmat;
				D3DXMatrixIdentity( &bmmat );
				CallF( boneptr->GetBoneMarkMatrix( &m_matWorld, &bmmat ), return 1 );

				g_pEffect->SetMatrix( g_hmWorld, &bmmat );
				bmarkptr->UpdateMatrix( &bmmat, &m_matVP );
				D3DXVECTOR4 difmult = D3DXVECTOR4( 0.25f, 0.5f, 0.5f, 0.5f );
				CallF( bmarkptr->OnRender( pdev, 0, difmult, 0, 0 ), return 1 );
			}
		}
	}

	if( bcircleptr ){
		map<int, CBone*>::iterator itrbone;
		for( itrbone = m_bonelist.begin(); itrbone != m_bonelist.end(); itrbone++ ){
			CBone* boneptr = itrbone->second;
			if( boneptr ){

				D3DXMATRIX bcmat;
				D3DXMatrixIdentity( &bcmat );
				CallF( boneptr->GetBoneCircleMatrix( &bcmat ), return 1 );

				D3DXMATRIX transmat = bcmat * m_matVP;
				D3DXVECTOR3 scpos;
				D3DXVec3TransformCoord( &scpos, &(boneptr->m_vertpos[BT_CHILD]), &transmat );
				scpos.z = 0.0f;
				bcircleptr->SetPos( scpos );
				D3DXVECTOR2 bsize( 0.025f, 0.025f );
				bcircleptr->SetSize( bsize );
				bcircleptr->SetColor( D3DXVECTOR4( 1.0f, 0.0f, 0.0f, 0.7f ) );
				CallF( bcircleptr->OnRender(), return 1 );

			}
		}
	}

	pdev->SetRenderState( D3DRS_ZFUNC, D3DCMP_LESSEQUAL );

	return 0;
}

