#include <windows.h>
#include <quaternion.h>
#include <math.h>
#include <crtdbg.h>


CQuaternion::CQuaternion()
{
	w = 1.0f;
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
}

CQuaternion::CQuaternion( float srcw, float srcx, float srcy, float srcz )
{
	w = srcw;
	x = srcx;
	y = srcy;
	z = srcz;
}
CQuaternion::~CQuaternion()
{

}

int CQuaternion::SetParams( float srcw, float srcx, float srcy, float srcz )
{
	w = srcw;
	x = srcx;
	y = srcy;
	z = srcz;
	return 0;
}

int CQuaternion::SetAxisAndRot( D3DXVECTOR3 srcaxis, float phai )
{
	float phai2;
	float cos_phai2, sin_phai2;

	phai2 = phai * 0.5f;
	cos_phai2 = cosf( phai2 );
	sin_phai2 = sinf( phai2 );

	w = cos_phai2;
	x = srcaxis.x * sin_phai2;
	y = srcaxis.y * sin_phai2;
	z = srcaxis.z * sin_phai2;

	return 0;
}
int CQuaternion::SetAxisAndRot( D3DXVECTOR3 srcaxis, double phai )
{
	double phai2;
	double cos_phai2, sin_phai2;

	phai2 = phai * 0.5;
	cos_phai2 = cos( phai2 );
	sin_phai2 = sin( phai2 );

	w = (float)cos_phai2;
	x = (float)( srcaxis.x * sin_phai2 );
	y = (float)( srcaxis.y * sin_phai2 );
	z = (float)( srcaxis.z * sin_phai2 );

	return 0;
}

int CQuaternion::GetAxisAndRot( D3DXVECTOR3* axisvecptr, float* frad )
{
	D3DXQUATERNION tempq;

	tempq.x = x;
	tempq.y = y;
	tempq.z = z;
	tempq.w = w;

	D3DXQuaternionToAxisAngle( &tempq, axisvecptr, frad );
	
	D3DXVec3Normalize( axisvecptr, axisvecptr );

	return 0;
}


int CQuaternion::SetRotation( D3DXVECTOR3 srcdeg )
{
	// ZAXAY̏ԂŁA]ANH[^jIZbgB

	CQuaternion q, qx, qy, qz;
	float cosx, sinx, cosy, siny, cosz, sinz;
	float fDeg2Pai = (float)DEG2PAI;

	cosx = (float)cos( srcdeg.x * 0.5f * fDeg2Pai );
	sinx = (float)sin( srcdeg.x * 0.5f * fDeg2Pai );
	cosy = (float)cos( srcdeg.y * 0.5f * fDeg2Pai );
	siny = (float)sin( srcdeg.y * 0.5f * fDeg2Pai );
	cosz = (float)cos( srcdeg.z * 0.5f * fDeg2Pai );
	sinz = (float)sin( srcdeg.z * 0.5f * fDeg2Pai );

	qx.SetParams( cosx, sinx, 0.0f, 0.0f );
	qy.SetParams( cosy, 0.0f, siny, 0.0f );
	qz.SetParams( cosz, 0.0f, 0.0f, sinz );

	q = qy * qx * qz;

	*this = q;

	return 0;
}

int CQuaternion::SetRotation( double degx, double degy, double degz )
{
	// ZAXAY̏ԂŁA]ANH[^jIZbgB

	CQuaternion q, qx, qy, qz;
	float cosx, sinx, cosy, siny, cosz, sinz;

	cosx = (float)cos( degx * 0.5 * DEG2PAI );
	sinx = (float)sin( degx * 0.5 * DEG2PAI );
	cosy = (float)cos( degy * 0.5 * DEG2PAI );
	siny = (float)sin( degy * 0.5 * DEG2PAI );
	cosz = (float)cos( degz * 0.5 * DEG2PAI );
	sinz = (float)sin( degz * 0.5 * DEG2PAI );

	qx.SetParams( cosx, sinx, 0.0f, 0.0f );
	qy.SetParams( cosy, 0.0f, siny, 0.0f );
	qz.SetParams( cosz, 0.0f, 0.0f, sinz );

	q = qy * qx * qz;

	*this = q;

	return 0;
}





CQuaternion CQuaternion::operator* (float srcw) const { return CQuaternion(this->w * srcw, this->x * srcw, this->y * srcw, this->z * srcw); }
CQuaternion &CQuaternion::operator*= (float srcw) { *this = *this * srcw; return *this; }
CQuaternion CQuaternion::operator/ (float srcw) const { return CQuaternion(this->w / srcw, this->x / srcw, this->y / srcw, this->z / srcw); }
CQuaternion &CQuaternion::operator/= (float srcw) { *this = *this / srcw; return *this; }
CQuaternion CQuaternion::operator+ (const CQuaternion &q) const { return CQuaternion(w + q.w, x + q.x, y + q.y, z + q.z); }
CQuaternion &CQuaternion::operator+= (const CQuaternion &q) { *this = *this + q; return *this; }
CQuaternion CQuaternion::operator- (const CQuaternion &q) const { return CQuaternion(w - q.w, x - q.x, y - q.y, z - q.z); }
CQuaternion &CQuaternion::operator-= (const CQuaternion &q) { *this = *this - q; return *this; }
CQuaternion CQuaternion::operator* (const CQuaternion &q) const {
	return CQuaternion(
		w * q.w - x * q.x - y * q.y - z * q.z,
		w * q.x + q.w * x + y * q.z - z * q.y,
		w * q.y + q.w * y + z * q.x - x * q.z,
		w * q.z + q.w * z + x * q.y - y * q.x ).normalize();
}
CQuaternion &CQuaternion::operator*= (const CQuaternion &q) { *this = *this * q; return *this; }
CQuaternion CQuaternion::operator- () const { return *this * -1.0f; }
CQuaternion CQuaternion::normalize () const { 
	float mag = w*w+x*x+y*y+z*z;
	if( mag != 0.0f )
		return (*this)*(1.0f/(float)::sqrt(mag));
	else
		return CQuaternion( 1e6, 1e6, 1e6, 1e6 );
}


D3DXMATRIX CQuaternion::MakeRotMatX()
{
	float dat00, dat01, dat02;
	float dat10, dat11, dat12;
	float dat20, dat21, dat22;

	dat00 = w * w + x * x - y * y - z * z;
	dat01 = 2.0f * ( x * y + w * z );
	dat02 = 2.0f * ( x * z - w * y );

	dat10 = 2.0f * ( x * y - w * z );
	dat11 = w * w - x * x + y * y - z * z;
	dat12 = 2.0f * ( y * z + w * x );

	dat20 = 2.0f * ( x * z + w * y );
	dat21 = 2.0f * ( y * z - w * x );
	dat22 = w * w - x * x - y * y + z * z;

	D3DXMATRIX retmat(
		dat00, dat01, dat02, 0.0f,
		dat10, dat11, dat12, 0.0f,
		dat20, dat21, dat22, 0.0f,
		0.0f, 0.0f, 0.0f, 1.0f );

	return retmat;
}


float CQuaternion::DotProduct( CQuaternion srcq )
{
	float dot;
	dot = w * srcq.w + 
		x * srcq.x + 
		y * srcq.y +
		z * srcq.z;
	return dot;
}

float CQuaternion::CalcRad( CQuaternion srcq )
{
	float dot, retrad;
	dot = this->DotProduct( srcq );

	//!!!!!!!!!!@Ӂ@!!!!!!!!!!!!
	//!!!! dot Pɑ傫l̂ƂAkakuɂ́AȒl(-1.#IN00)Ă܂B
	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if( dot > 1.0f )
		dot = 1.0f;
	if( dot < -1.0f )
		dot = -1.0f;
	retrad = (float)acos( dot );

	return retrad;
}


CQuaternion CQuaternion::Slerp( CQuaternion endq, int framenum, int frameno )
{
	CQuaternion retq;
	retq.SetParams( 1.0f, 0.0f, 0.0f, 0.0f );

	float kaku;
	kaku = this->CalcRad( endq );

	if( kaku > (PI * 0.5f) ){
		//Е-qɂ΁A(PI * 0.5f)菬ȂBiŒZR[Xǂj
		endq = -endq;
		kaku = this->CalcRad( endq );
		_ASSERT( kaku <= (PI * 0.5f) );
	}

	// sin( kaku ) == 0.0 t߂𒲐B
	//180xɂ͂ȂȂ̂ŁiȂȂ悤ɂ̂ŁjOx̂݃PA
	int kaku0flag = 0;
	if( (kaku <= 1e-4) && (kaku >= -1e-4) ){
		kaku0flag = 1;
//DbgOut( "Quaternion : Slerp : kaku0flag 1 : dot %f, kaku %f\n", dot, kaku );
	}


	float t = (float)frameno / (float)framenum;
	float alpha, beta;
	if( kaku0flag == 0 ){
		alpha = (float)sin( kaku * (1.0f - t) ) / (float)sin( kaku );
		beta = (float)sin( kaku * t ) / (float)sin( kaku );

		retq = *this * alpha + endq * beta;
	}else{
		retq = *this;
//DbgOut( "MotionInfo : FillUpMotionPoint 2: frame %d, %f, %f, %f, %f\n",
//	   frameno, startq.w, startq.x, startq.y, startq.z );
	}
		
	return retq;
}


int CQuaternion::Squad( CQuaternion q0, CQuaternion q1, CQuaternion q2, CQuaternion q3, float t )
{

	CQuaternion iq0, iq1, iq2, iq3;
	D3DXQUATERNION qx0, qx1, qx2, qx3;
	D3DXQUATERNION ax, bx, cx, resx;

	q0.inv( &iq0 );
	q1.inv( &iq1 );
	q2.inv( &iq2 );
	q3.inv( &iq3 );

	Q2X( &qx0, iq0 );
	Q2X( &qx1, iq1 );
	Q2X( &qx2, iq2 );
	Q2X( &qx3, iq3 );

	D3DXQuaternionSquadSetup( &ax, &bx, &cx, &qx0, &qx1, &qx2, &qx3 );
	D3DXQuaternionSquad( &resx, &qx1, &ax, &bx, &cx, t );

	D3DXQUATERNION iresx;
	D3DXQuaternionInverse( &iresx, &resx );

	this->x = iresx.x;
	this->y = iresx.y;
	this->z = iresx.z;
	this->w = iresx.w;


	return 0;
}


int CQuaternion::Q2X( D3DXQUATERNION* dstx )
{
	dstx->x = x;
	dstx->y = y;
	dstx->z = z;
	dstx->w = w;
	
	return 0;
}

int CQuaternion::Q2X( D3DXQUATERNION* dstx, CQuaternion srcq )
{
	dstx->x = srcq.x;
	dstx->y = srcq.y;
	dstx->z = srcq.z;
	dstx->w = srcq.w;
	
	return 0;
}

int CQuaternion::inv( CQuaternion* dstq ) 
{

	dstq->w = w;	
	dstq->x = -x;
	dstq->y = -y;
	dstq->z = -z;

	*dstq = dstq->normalize();

	return 0;
}

int CQuaternion::RotationArc( D3DXVECTOR3 srcvec0, D3DXVECTOR3 srcvec1 )
{
	//srcvec0, srcvec1́AnormalizeĂƂB

	D3DXVECTOR3 c;
	D3DXVec3Cross( &c, &srcvec0, &srcvec1 );
	float d;
	d = D3DXVec3Dot( &srcvec0, &srcvec1 );
	float s;
	s = (float)sqrt( (1 + d) * 2.0f );

	x = c.x / s;
	y = c.y / s;
	z = c.z / s;
	w = s / 2.0f;

	return 0;
}


int CQuaternion::Rotate( D3DXVECTOR3* dstvec, D3DXVECTOR3 srcvec )
{
	D3DXMATRIX mat;

	mat = MakeRotMatX();

	D3DXVec3TransformCoord( dstvec, &srcvec, &mat );
	
	return 0;
}

int CQuaternion::QuaternionToAxisAngle( D3DXVECTOR3* dstaxis, float* dstrad )
{
	D3DXQUATERNION xq;

	int ret;
	ret = Q2X( &xq );
	_ASSERT( !ret );

	D3DXQuaternionToAxisAngle( &xq, dstaxis, dstrad );

	return 0;
}

int CQuaternion::transpose( CQuaternion* dstq )
{
	D3DXMATRIX matx;

	matx = MakeRotMatX();

	D3DXMATRIX tmatx;
	D3DXMatrixTranspose( &tmatx, &matx );

	D3DXQUATERNION qx;
	D3DXQuaternionRotationMatrix( &qx, &tmatx );

	dstq->x = qx.x;	
	dstq->y = qx.y;	
	dstq->z = qx.z;	
	dstq->w = qx.w;	

	return 0;
}

